--[[
    Database of all minipets and mounts
    $Revision: 77413 $
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "PetAndMountDatabase-1.0";
local MINOR_VERSION = "$Rev: 77413 $";

local PetAndMountDatabase, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION );
if( not PetAndMountDatabase ) then
    return;
end

local pt3 = LibStub( "LibPeriodicTable-3.1", true );

if( not pt3 ) then
    error( MAJOR_VERSION .. " requires LibPeriodicTable-3.1" );
end

pt3:AddData( "PetAndMountDatabase", PetAndMountDatabaseSets.Revision, PetAndMountDatabaseSets.Sets );
    
--[[
    Value meanings:
    
    Mounts.*: Maximum speed increase over running in percent, where running is 100%
    MiniPets.Reagented: ItemID of item needed to summon (snowball)
    MiniPets.Quest: QuestID of quest item is used for
    MiniPets.Equipment: SlotName of slot the equipment goes in
    
    Negative number is a SPELL instead of an ITEM
]]--

PetAndMountDatabase.Version = tostring( PetAndMountDatabaseSets.WoWVersion ); -- Matches the WoW version number this is accurate for

-- Backwards compatibility stuff
PetAndMountDatabase.Collections = {
        MiniPets = {
                Normal = "MiniPets.Normal",
                Holiday = "MiniPets.Reagented.Snowball",
                Equipment = "MiniPets.Equipment",
                Children = "MiniPets.Children",
                Quest = "MiniPets.Quest",
                Reagented = "MiniPets.Reagented",
            }, -- MiniPets

        Mounts = {
                Ground = {
                        Slow = "Mounts.Ground.Slow",
                        Fast = "Mounts.Ground.Fast",
                        VeryFast = "Mounts.Ground.VeryFast",
                    }, -- Ground
                Flying = {
                        Fast = "Mounts.Flying.Fast",
                        VeryFast = "Mounts.Flying.VeryFast",
                        ExtremelyFast = "Mounts.Flying.ExtremelyFast",
                    }, -- Flying
            }, -- Mounts
    }; -- Collections
    
PetAndMountDatabase.FoundSets = {};
PetAndMountDatabase.SubSets = {};

function PetAndMountDatabase.NormalizeSetName( set )
    if( set:find( "PetAndMountDatabase." ) ~= 1 ) then
        set = "PetAndMountDatabase." .. set;
    end--if
    if( set == "PetAndMountDatabase.MiniPets.Holiday" ) then
        set = "PetAndMountDatabase.MiniPets.Reagented.Snowball";
    end--if
    return set;
end--PetAndMountDatabase.NormalizeSetName( set )

function PetAndMountDatabase:HasSet( set )
    local sets = pt3.sets;

    set = PetAndMountDatabase.NormalizeSetName( set );
    if( self.FoundSets[set] ) then
        return true;
    end--if
    for setName, _ in pairs( sets ) do
        if( setName:find( set ) == 1 ) then
            self.FoundSets[set] = true;
            return true;
        end--if
    end--if
    return false;
end--PetAndMountDatabase:HasSet( set )

function PetAndMountDatabase:GetSubsets( set )
    local sets = pt3.sets;

    set = PetAndMountDatabase.NormalizeSetName( set );
    if( self.SubSets[set] ) then
        return self.SubSets[set];
    end--if
    set = set .. ".";
    for setName, _ in pairs( sets ) do
        local start, ends = setName:find( set );

        if( ( start == 1 ) and ( ends ~= setName:len() ) ) then
            setName = setName:sub( ends + 1 );
            if( setName:find( "." ) ) then
                setName = ("%s%s"):format( set, strsplit( ".", setName ) );
            end--if
            if( self.SubSets[set] == nil ) then
                self.SubSets[set] = {};
            end--if
            self.SubSets[set][setName] = setName;
        end--if
    end--if
    return self.SubSets[set];
end--PetAndMountDatabase:GetSubsets( set )

function PetAndMountDatabase:ItemInSet( item, set )
    if( item == nil ) then
        return false;
    end--if
    set = PetAndMountDatabase.NormalizeSetName( set );
    return pt3:ItemInSet( item, set );
end--PetAndMountDatabase:ItemInSet( item, set )

function PetAndMountDatabase:SpellInSet( spellID, set )
    if( spellID == nil ) then
        return false;
    end--if
    if( spellID > 0 ) then
        spellID = spellID * -1;
    end--if
    return self:ItemInSet( spellID, set );
end--PetAndMountDatabase:SpellInSet( item, set )

function PetAndMountDatabase:IterateSet( set )
    set = PetAndMountDatabase.NormalizeSetName( set );
    return pt3:IterateSet( set );
end--PetAndMountDatabase:IterateSet( set )

PetAndMountDatabaseSets = nil;
